/*
    This file is part of AutoQuad.

    AutoQuad is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    AutoQuad is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with AutoQuad.  If not, see <http://www.gnu.org/licenses/>.

    Copyright ©  2013 Frederic Guichard
	Base on file futaba.c
*/

#include "aq.h"
#include "radio.h"
#include "grhott.h"
#include "notice.h"
#include "aq_mavlink.h"
#include "imu.h"
#include "util.h"
#include <string.h>

grhottStruct_t grhott_Data __attribute__((section(".ccm")));

int grhottDecode(void) {
	// calcul CRC16 XMODEM
	int crc = 0xA604 ;									/* CRC  3 premieres valeurs = A80108 */
	for (int num=0; num < grhott_Data.nb_channel - 2  ; num++)               /* Step through bytes in memory */
	{
		crc = crc ^ (grhott_Data.rawBuf[num] << 8);     /* Fetch byte from memory, XOR into CRC top byte*/
		for (int i=0; i<8; i++)              			/* Prepare to rotate 8 bits */
		{
			if (crc & 0x8000)              				/* b15 is set... */
				crc = (crc << 1) ^ POLYNOM;     			/* rotate and XOR with XMODEM polynomic */
			else                           				/* b15 is clear... */
				crc = crc << 1;              			/* just rotate */
		}                              					/* Loop for 8 bits */
	}
	int grhott_crc = ( grhott_Data.rawBuf[16] << 8 ) + grhott_Data.rawBuf[17] ;
	if (crc == grhott_crc) {
		radioData.channels[0] = ( int16_t ) ( ( ( grhott_Data.rawBuf[0] << 8 ) + grhott_Data.rawBuf[1] - GRHOTT_MIN ) / 5 );
		radioData.channels[1] = ( int16_t ) ( ( ( grhott_Data.rawBuf[2] << 8 ) + grhott_Data.rawBuf[3] - GRHOTT_MID ) / 5 );
		radioData.channels[2] = ( int16_t ) ( ( ( grhott_Data.rawBuf[4] << 8 ) + grhott_Data.rawBuf[5] - GRHOTT_MID ) / 5 );
		radioData.channels[3] = ( int16_t ) ( ( ( grhott_Data.rawBuf[6] << 8 ) + grhott_Data.rawBuf[7] - GRHOTT_MID ) / 5 );
		radioData.channels[4] = ( int16_t ) ( ( ( grhott_Data.rawBuf[8] << 8 ) + grhott_Data.rawBuf[9] - GRHOTT_MID ) / 5 );
		radioData.channels[5] = ( int16_t ) ( ( ( grhott_Data.rawBuf[10] << 8 ) + grhott_Data.rawBuf[11] - GRHOTT_MID ) / 5 );
		radioData.channels[6] = ( int16_t ) ( ( ( grhott_Data.rawBuf[12] << 8 ) + grhott_Data.rawBuf[13] - GRHOTT_MID ) / 5 );
		radioData.channels[7] = ( int16_t ) ( ( ( grhott_Data.rawBuf[14] << 8 ) + grhott_Data.rawBuf[15] - GRHOTT_MID ) / 5 );
		radioData.channels[8] = 0 ;
		radioData.channels[9] = 0 ;
		radioData.channels[10] = 0 ;
		radioData.channels[11] = 0 ;
		radioData.channels[12] = 0 ;
		return 1;
    } else {
		static char s[40];
		sprintf(s, "GRHOTT CRC ERREUR : %x != %x  ?",grhott_crc,crc);
		AQ_NOTICE(s);	
		return 0;
	}
}

int grhottCharIn(unsigned char c) {
    switch (grhott_Data.state) {
    case GRHOTT_WAIT_SYNC1:
	if (c == GRHOTT_START_CHAR1) {
	    grhott_Data.state = GRHOTT_WAIT_SYNC2;
	    grhott_Data.dataCount = 0;
	    grhott_Data.nb_channel = 0 ;
	}
	break;

    case GRHOTT_WAIT_SYNC2:
	if (c == GRHOTT_START_CHAR2) {
	    grhott_Data.state = GRHOTT_WAIT_NB_CHANNEL;
	}
	break;

    case GRHOTT_WAIT_NB_CHANNEL:
	grhott_Data.nb_channel = c * 2 + 2 ;
	grhott_Data.state = GRHOTT_WAIT_DATA;
	grhott_Data.dataCount = 0;
	break;

    case GRHOTT_WAIT_DATA:
	grhott_Data.rawBuf[grhott_Data.dataCount++] = c;
	if (grhott_Data.dataCount == grhott_Data.nb_channel ) {
	    grhott_Data.state = GRHOTT_WAIT_SYNC1;
	    return grhottDecode();
	}
	break;

    return 0;
    }
}

void grhottInit(void) {
    USART_InitTypeDef USART_InitStructure;

    memset((void *)&grhott_Data, 0, sizeof(grhott_Data));

    radioData.serialPort = serialOpen(GRHOTT_UART, GRHOTT_BAUD, USART_HardwareFlowControl_None, GRHOTT_RXBUF_SIZE, 0);

    // modify default serial configuration
    USART_StructInit(&USART_InitStructure);
    USART_InitStructure.USART_BaudRate = GRHOTT_BAUD;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx;
    USART_Init(GRHOTT_UART, &USART_InitStructure);

    USART_Cmd(GRHOTT_UART, ENABLE);
	
	grhott_Data.state = GRHOTT_WAIT_SYNC1;

}
